class CorpusFileList extends RessourceList {
  constructor(listElement, options = {}) {
    super(listElement, {...CorpusFileList.options, ...options});
    this.corpusId = listElement.dataset.corpusId;
    this.user.eventListeners.corpus.addEventListener((eventType, payload) => this.eventHandler(eventType, payload), this.corpusId);
  }

  init() {
    super.init(this.user.data.corpora[this.corpusId].files);
  }

  onclick(event) {
    let ressourceElement = event.target.closest('tr');
    if (ressourceElement === null) {return;}
    let corpusFileId = ressourceElement.dataset.id;
    let actionButtonElement = event.target.closest('.action-button');
    if (actionButtonElement === null) {return;}
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete':
        let deleteModalHTML = `<div class="modal">
                                 <div class="modal-content">
                                   <h4>Confirm corpus deletion</h4>
                                   <p>Do you really want to delete the corpus file <b>${this.user.data.corpora[this.corpusId].files[corpusFileId].filename}</b>? It will be permanently deleted!</p>
                                 </div>
                                 <div class="modal-footer">
                                   <a href="#!" class="btn modal-close waves-effect waves-light">Cancel</a>
                                   <a class="btn modal-close red waves-effect waves-light" href="${this.user.data.corpora[this.corpusId].files[corpusFileId].url}/delete"><i class="material-icons left">delete</i>Delete</a>
                                 </div>
                               </div>`;
        let deleteModalParentElement = document.querySelector('#modals');
        deleteModalParentElement.insertAdjacentHTML('beforeend', deleteModalHTML);
        let deleteModalElement = deleteModalParentElement.lastChild;
        let deleteModal = M.Modal.init(deleteModalElement, {onCloseEnd: () => {deleteModal.destroy(); deleteModalElement.remove();}});
        deleteModal.open();
        break;
      case 'download':
        window.location.href = this.user.data.corpora[this.corpusId].files[corpusFileId].download_url;
        break;
      case 'view':
        if (corpusFileId !== '-1') {window.location.href = this.user.data.corpora[this.corpusId].files[corpusFileId].url;}
        break;
      default:
        console.error(`Unknown action: "${action}"`);
        break;
    }
  }

  patch(patch) {
    let id, match, re, valueName;
    for (let operation of patch) {
      switch(operation.op) {
        case 'add':
          // Matches the only paths that should be handled here: /corpora/{this.user.data.corpora[this.corpusId].id}/files/{corpusFileId}
          re = new RegExp('^/corpora/' + this.user.data.corpora[this.corpusId].id + '/files/(\\d+)$');
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        case 'remove':
          // See case add ;)
          re = new RegExp('^/corpora/' + this.user.data.corpora[this.corpusId].id + '/files/(\\d+)$');
          if (re.test(operation.path)) {
            [match, id] = operation.path.match(re);
            this.remove(id);
          }
          break;
        case 'replace':
          // Matches the only paths that should be handled here: /corpora/{corpusId}/{status || description || title}
          re = new RegExp('^/corpora/' + this.user.data.corpora[this.corpusId].id + '/files/(\\d+)/(author|filename|publishing_year|title)$');
          if (re.test(operation.path)) {
            [match, id, valueName] = operation.path.match(re);
            this.replace(id, valueName, operation.value);
          }
          break;
        default:
          break;
      }
    }
  }

  preprocessRessource(corpusFile) {
    return {id: corpusFile.id, author: corpusFile.author, filename: corpusFile.filename, publishing_year: corpusFile.publishing_year, title: corpusFile.title};
  }
}
CorpusFileList.options = {
  item: `<tr>
           <td><span class="filename"></span></td>
           <td><span class="author"></span></td>
           <td><span class="title"></span></td>
           <td><span class="publishing_year"></span></td>
           <td class="right-align">
             <a class="action-button btn-floating red tooltipped waves-effect waves-light" data-action="delete" data-position="top" data-tooltip="Delete"><i class="material-icons">delete</i></a>
             <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="download" data-position="top" data-tooltip="View"><i class="material-icons">file_download</i></a>
             <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="view" data-position="top" data-tooltip="View"><i class="material-icons">send</i></a>
           </td>
         </tr>`,
  valueNames: [{data: ['id']}, 'author', 'filename', 'publishing_year', 'title']
};
